@@ -1,6 +1,6 @@
 package Mail::DMARC::Base;
 {
-  $Mail::DMARC::Base::VERSION = '0.20130528';
+  $Mail::DMARC::Base::VERSION = '1.20130528';
 }
 use strict;
 use warnings;
@@ -179,6 +179,19 @@ sub is_valid_domain {
     return 0;
 }
 
+sub is_valid_spf_scope {
+    my ($self, $scope ) = @_;
+    return lc $scope if grep { lc $scope eq $_ } qw/ mfrom helo /;
+    return;
+};
+
+sub is_valid_spf_result {
+    my ($self, $result ) = @_;
+    return 1 if grep { lc $result eq $_ }
+        qw/ fail neutral none pass permerror softfail temperror /;
+    return;
+};
+
 sub slurp {
     my ( $self, $file ) = @_;
     open my $FH, '<', $file or croak "unable to read $file: $!";
@@ -204,7 +217,7 @@ Mail::DMARC::Base - DMARC utility functions
 
 =head1 VERSION
 
-version 0.20130528
+version 1.20130528
 
 =head1 METHODS
 
